#ifndef CDEPENDENCIES_H
#define CDEPENDENCIES_H
#include <gtkmm.h>
#include <vector>

// This class takes a list of packages to be installed, and using a
// database of all packages it adds the dependencies recursively.  It
// also presents the user with a dialog if things go wrong.
// See COPYING for license information.

class CDependencies
{
public:
	CDependencies(std::vector<struct_package>& pkglist, packagemap_t& packages);
protected:
	void exportDeps(Glib::ustring deps);
	void removeDeps(Glib::ustring deps, std::vector<struct_package>& pkglist);
};

CDependencies::CDependencies(std::vector<struct_package>& pkglist, packagemap_t& packages)
{	// Recursively add dependencies for packages.
	bool		_depsfound = true;
	Glib::ustring	_fails;
	while ( _depsfound ) {
		_depsfound = false;
		for ( gint32 i = 0; i < pkglist.size(); i++ ) {
			Glib::ustring	_strdeps = pkglist[i].deps;
			while ( (gint32)_strdeps.find(',') > 0 ) {
				Glib::ustring	_pkg = _strdeps.substr(0, _strdeps.find(','));
				_strdeps = _strdeps.substr(_strdeps.find(',') + 1);
				if (	packages.find(_pkg) != packages.end() &&
						!packages[_pkg].installedversion.length() &&
						packages[_pkg].mirrorversion.length() &&
						packages[_pkg].color != COLOR_BLACKLIST) {
					bool	_found = false;
					for ( gint32 k = 0; k < pkglist.size(); k++ )
						if ( pkglist[k].name == _pkg ) _found = true;
					if ( !_found ) {
						pkglist.push_back(packages[_pkg]);
						_depsfound = true;
					}
				} else if ( (packages.find(_pkg) == packages.end()) && (int)_fails.find(pkglist[i].name + ":  " + _pkg) == -1 )
					_fails += "\n\tFor " + pkglist[i].name + ":  " + _pkg; 
			}
		}
	}

	if ( _fails.length() > 0 ) {	// There were FAILs. n00b.
		_fails = "The following dependencies could not be found:" + _fails;
		Gtk::Dialog	_dialog;
		Gtk::Label	_label;
		_dialog.get_vbox()->pack_start(_label);
		_dialog.set_title("Failed dependencies");
		_dialog.set_border_width(6);
		_dialog.add_button("_Ignore", 1);
		_dialog.add_button("_Export", 2);
		_dialog.add_button("_Don't install", 3);
		_dialog.set_default_response(1);
		_label.set_text(_fails);
		_dialog.show_all_children();
		// If the user clicks export, run the UI again so they
		// can choose ignore or don't install.
		while ( int _result = _dialog.run() ) {
			if ( _result == 2 ) exportDeps(_fails);
			if ( _result == 3 ) removeDeps(_fails, pkglist);
			if ( _result != 2 ) break;
		}
	}
}

void CDependencies::exportDeps(Glib::ustring deps)
{	// Choose a file and export the deps to that file.
	Gtk::FileChooserDialog	_dlg("Export dependency file", Gtk::FILE_CHOOSER_ACTION_SAVE);
	_dlg.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
	_dlg.add_button(Gtk::Stock::SAVE, Gtk::RESPONSE_OK);
	if ( _dlg.run() != Gtk::RESPONSE_OK ) return;
	_dlg.hide();
	Glib::ustring	_filename = _dlg.get_filename();
	if ( (int)_filename.rfind('/') > (int)_filename.rfind('.') )
		_filename += ".dep";

	// Make the list comma-seperated.
	deps = deps.substr(deps.find(":  ") + 3);
	while ( (int)deps.find('\n') >= 0 )
		deps.replace(deps.find('\n'), deps.find(":  ") - deps.find('\n') + 3, ",");

	std::ofstream	_ofile;
	_ofile.open(_filename.c_str(), std::ios_base::out);
	_ofile << deps << "\n";
	_ofile.close();
}

void CDependencies::removeDeps(Glib::ustring deps, std::vector<struct_package>& pkglist)
{	// Remove the unmet deps from the pending list, so that they don't get isntalled.
	deps = deps.substr(deps.find('\n') + 6) + "\n      ";
	while ( (int)deps.find(":  ") >= 0 ) {
		for ( int i = 0; i < pkglist.size(); i++ ) {
			if ( pkglist[i].name == deps.substr(0, deps.find(":  ")) ) {
				pkglist.erase(pkglist.begin() + i);
				i--;
			}
		}
		deps = deps.substr(deps.find('\n') + 6);
	}
}

#endif
