#ifndef COMMON_FUN_H
#define COMMON_FUN_H
namespace Common {

#include <gtkmm.h>
#include <sstream>

// These are extremely simple functions used by multiple classes, so
// they have been declared here to be included and used by any class.
// See COPYING for license information.

struct struct_package {
	Glib::ustring	name;
	Glib::ustring	location;
	guint32		packedsize;
	guint32		unpackedsize;
	Glib::ustring	deps;
	Glib::ustring	cons;
	Glib::ustring	installedversion;
	Glib::ustring	mirrorversion;
	Glib::ustring	color;
	std::vector<Glib::ustring>	description;
	Glib::ustring	type;	// Filetype (.tgz, .tlz, etc.)
};
typedef std::map<Glib::ustring, struct_package>	packagemap_t;

#define COLOR_INSTALL	"#008A00"
#define COLOR_REMOVE	"#FF5050"
#define COLOR_NEUTRAL	"#FFFFFF"
#define COLOR_BLACKLIST	"#000000"

Glib::ustring GetPackageName(const Glib::ustring& fromstr)
{	// Returns just the package name from an entire string, i.e. "fghfgh" from "fghfgh-10.0.1-i486-62.1.tgz"
	guint8	_dashcount = 0;
	for ( gint16 i = fromstr.length() - 1; i >= 0; i-- ) {
		if ( fromstr.substr(i, 1) == "-" ) {
			_dashcount++;
			if ( _dashcount < 3 ) continue;
			return fromstr.substr(0, i);
		}
	}
	if ( _dashcount > 0 ) return fromstr.substr(0, fromstr.find('-'));	// Helps out screwed up packages.
	return "";
}

Glib::ustring GetPackageVersion(const Glib::ustring& fromstr)
{	// Returns just the version from the package filename, i.e. "10.0.1-i486-62.1" from "fghfgh-10.0.1-i486-62.1.tgz"
	guint8	_dashcount = 0;

	for ( gint16 i = fromstr.length() - 1; i >= 0; i-- ) {
		if ( fromstr.substr(i, 1) == "-" ) {
			_dashcount++;
			if ( _dashcount == 3 )
				return fromstr.substr(i + 1, (fromstr.length() - i) - 5);
		}
	}
	// Fix for messed up packages like cupswrapperMFC that Atticus pointed out.
	if ( _dashcount > 0 ) return fromstr.substr(fromstr.find('-') + 1, (fromstr.length() - fromstr.find('-')) - 5);
	return "";
}

Glib::ustring LTrim(Glib::ustring str)
{	// This function trims spaces off the beginning of a string.
	while ( str.length() > 0 ) {
		if ( !str.find(' ') ) str.erase(0, 1);
		else break;
	}
	return str;
}

Glib::ustring RTrim(Glib::ustring str)
{	// Trim the spaces off the end of a string.
	while ( str.length() > 0 ) {
		if ( str.substr(str.length() - 1) == " " ) str.erase(str.length() - 1);
		else break;
	}
	return str;
}

Glib::ustring IntToString(int num)
{
	std::stringstream	_ss;
	Glib::ustring		_retval = "";
	_ss << num;
	_ss >> _retval;
	return _retval;
}

bool FileExists(const char* filename)
{
	std::ifstream	_ifile;
	_ifile.open(filename, std::ios_base::in);
	if ( !_ifile.is_open() ) return false;
	else _ifile.close();
	return true;
}


}
#endif
